import game_services
import services
import random
import MATY
from MATY import inject_to, error

def log(txt): MATY.log(txt, 'mscheck.txt')

from developmental_milestones.developmental_milestone_tracker import DevelopmentalMilestoneTracker, MilestoneTelemetryContext
@inject_to(DevelopmentalMilestoneTracker, 'on_lod_update')
def on_lod_update_nodelete(original, self, old_lod, new_lod, *args, **kwargs):
    if new_lod < self._tracker_lod_threshold:
        pass
    elif old_lod < self._tracker_lod_threshold:
        original(self, old_lod, new_lod, *args, **kwargs)
    if self._sim_info.is_npc:
        self.load_milestones_info_from_proto()
        self.send_all_milestones_update_to_client()
    return

from zone_types import ZoneState
@inject_to(DevelopmentalMilestoneTracker, 'on_zone_load')
def on_zone_load(original, self, *args, **kwargs):
    if self._sim_info.is_npc:
        try:
            self.load_milestones_info_from_proto()
            services.current_zone().register_callback(ZoneState.ALL_SIMS_SPAWNED, self.start_milestone_tracker)
        except BaseException as e:
            error('Failed to load zone for NPC!', e, log)
            original(self, *args, **kwargs)
    else:
        original(self, *args, **kwargs)
    return

from protocolbuffers import GameplaySaveData_pb2
@inject_to(DevelopmentalMilestoneTracker, 'on_zone_unload')
def on_zone_unload(original, self, *args, **kwargs):
    if self._sim_info.is_npc:
        try:
            self._developmental_milestone_proto = GameplaySaveData_pb2.DevelopmentalMilestoneTrackerData()
            self.save_milestones_info_to_proto(self._developmental_milestone_proto, copy_existing=False)
        except BaseException as e:
            error('Failed to unload zone for NPC!', e, log)
    return original(self, *args, **kwargs)

from objects import ALL_HIDDEN_REASONS_EXCEPT_UNINITIALIZED
@inject_to(DevelopmentalMilestoneTracker, 'start_milestone_tracker')
def on_start_milestone_tracker(original, self, *args, **kwargs):
    if self._sim_info.is_npc:
        try:
            is_instanced = self._sim_info.is_instanced(allow_hidden_flags=ALL_HIDDEN_REASONS_EXCEPT_UNINITIALIZED)
            zone_is_running = services.current_zone().is_zone_running
            if is_instanced and self._setup_delayed_goals:
                self.setup_goals()
            self._activate_available_milestones(telemetry_context=MilestoneTelemetryContext.NEW_SIM, activate_goals=is_instanced)
            if not self._initial_loot_applied and is_instanced:
                if zone_is_running:
                    self._apply_retroactive_milestones_from_gameplay()
                else:
                    self._apply_initial_loot()
                    services.current_zone().register_callback(ZoneState.RUNNING, self._shutdown_retroactive_only_milestones)
            if not zone_is_running:
                services.current_zone().register_callback(ZoneState.RUNNING, self.send_all_milestones_update_to_client)
            return
        except BaseException as e:
            error('start_milestone_tracker error', e, log)
    return original(self, *args, **kwargs)
